'''
W.Wojnowski 2022
'''
# imports:
import tkinter as tk
from tkinter import ttk
from tkinter import messagebox
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.colors import LinearSegmentedColormap
from tkinter import filedialog
from math import log
from fpdf import FPDF
import os
from datetime import datetime

# start the main app:
root = tk.Tk()
root.title('AGREEprep - Analytical Greenness Metric for Sample Preparation')
root.geometry('760x450')
root.iconbitmap('AGREEPrep_icon.ico')
# fix the window dimensions:
root.resizable(0, 0)

# define the color palette. This can be based on https://flatuicolors.com
colors = {
    'foreground': '#ffffff',
    'background': '#dfe4ea',
    'accent': '#70a1ff',
    'text': 'black'
}

# configure the background:
root.configure(bg=colors['background'], padx=8, pady=8)

# create and configure a style to be used in tabs and labels
s = ttk.Style()
s.theme_create("AGREE_2.0_theme", parent="alt", settings={
    "TNotebook": {
        "configure": {
            'background': colors['background'],
            'borderwidth': 0,
            'tabmargins': [0, 0, 0, 0],
            'padding': [0, 0, 5, 0],
            'relief': 'flat'}},
    "TNotebook.Tab": {
        "configure": {
            'padding': [11, 1],
            'background': colors['accent'],
            'foreground': colors['text'],
            # change the colour of a dashed frame around the title of the
            # selected frame to the foreground colour, so that it is not visible:
            'focuscolor': colors['foreground'],
            'borderwidth': 0,
            'bordercolor': 'red'},
        "map": {
            "background": [("selected", colors['foreground'])],
            "expand": [("selected", [1, 1, 1, 0])]}},
    'TLabel': {
        'configure': {
            'background': colors['foreground'],
            'foreground': colors['text'],
            'padding': [5, 5, 5, 5]}},
    'TFrame': {
        'configure': {
            'background': colors['foreground'],
            'borderwidth': 0}},
    'TEntry': {
        'configure': {
            'background': 'red',
            'fieldbackground': colors['background'],
            'foreground': colors['text'],
            'insertcolor': colors['text'],
            'selectbackground': colors['accent'],
        }},
    'TButton': {
        'configure': {
            'background': colors['accent'],
            'focuscolor': colors['accent'],
            'foreground': colors['text'],
            'anchor': 'center'},
        'map': {
            'relief': [('pressed', 'groove')]
        }},
    'TRadiobutton': {
        'configure': {
            'focuscolor': colors['foreground'],
            'background': colors['foreground'],
            'foreground': colors['text'],
            'indicatorcolor': colors['background']},
        "map": {
            "background": [("active", colors['accent']),
                           ("pressed", colors['accent']),
                           ("disabled", colors['accent'])],
            "foreground": [("active", colors['text'])],
            'indicatorcolor': [('selected', colors['accent'])]
        }},
    'TCheckbutton': {
        'configure': {
            'focuscolor': colors['foreground'],
            'background': colors['foreground'],
            'foreground': colors['text'],
            'indicatorcolor': colors['background']},
        "map": {
            "background": [("active", colors['accent']),
                           ("pressed", colors['accent']),
                           ("disabled", colors['accent'])],
            "foreground": [("active", colors['text'])],
            'indicatorcolor': [('selected', colors['accent'])]
        }},
    'TMenubutton': {
        'configure': {
            'background': colors['accent'],
            'foreground': colors['text'],
            'width': '20'
        }},
    'TMenu': {
        'configure': {
            'background': colors['accent'],
            'foreground': colors['text'],
            'width': '20',
            'arrowcolor': colors['text'],
        }},
})

s.theme_use("AGREE_2.0_theme")

# to remove relief from the TEntry widget
# based on https://stackoverflow.com/questions/44383730/
s.layout('AGREE_2.0_theme', [
    ('Entry.highlight', {
        'sticky': 'nswe',
        'children':
            [('Entry.border', {
                'border': '1',
                'sticky': 'nswe',
                'children':
                    [('Entry.padding', {
                        'sticky': 'nswe',
                        'children':
                            [('Entry.textarea',
                              {'sticky': 'nswe'})]
                    })]
            })]
    })])

s.configure('AGREE_2.0_theme', relief='flat',
            background=colors['background'],
            foreground=colors['text'],
            insertcolor=colors['text'],
            selectbackground=colors['accent']
            )


def call_info_popup():
    win = tk.Toplevel()
    win_frame = tk.Frame(win, width=500, background=colors['foreground'])
    win_frame.pack()
    win.iconbitmap('AGREEPrep_icon.ico')
    win.wm_title('About and citation info')
    text = ttk.Label(win_frame, text='If you use AGREEprep, please cite:', wraplength=280, justify='left')
    text.grid(column=0, row=0, padx=8, pady=8, sticky='w')
    text2 = tk.Text(win_frame, width=50, height=5, wrap='word', bg=colors['background'])
    citation = 'W.Wojnowski, M.Tobiszewski, F.Pena-Pereira, E.Psillakis, AGREEprep - Analytical Greenness Metric' \
               ' for Sample Preparation, TrAC Trends in Analytical Chemistry (2022), doi.org/10.1016/j.trac.2022.116553'
    text2.grid(column=0, row=1, padx=8, pady=8, sticky='w')
    text2.insert(tk.END, citation)
    text3 = ttk.Label(win_frame, text='Most recent version of the source code can be found at:', wraplength=320, justify='left')
    text3.grid(column=0, row=2, padx=8, pady=8, sticky='w')
    text4 = tk.Text(win_frame, width=50, height=1, wrap='word', bg=colors['background'])
    source_link = 'git.pg.edu.pl/p174235/agreeprep'
    text4.grid(column=0, row=3, padx=8, pady=8, sticky='w')
    text4.insert(tk.END, source_link)
    text5 = ttk.Label(win_frame, text='Copyright (c) 2022 W.Wojnowski, M.Tobiszewski, F.Pena-Pereira, E.Psillakis, '
                                      'available under the MIT License', wraplength=380,
                      justify='left')
    text5.grid(column=0, row=4, padx=8, pady=8, sticky='w')

def save_image():
    ftypes = [('PNG file', '.png'), ('SVG file', '.svg'), ('All files', '*')]
    filename = filedialog.asksaveasfilename(filetypes=ftypes, defaultextension='.png')
    # save the plot in the specified path; the 'tight' option removes the whitespace from around the figure:
    plt.savefig(filename, bbox_inches='tight', dpi=300)


def reset_scores():
    for criterion in criteria:
        criterion.textvar.set('1.0')
        criterion.optionvar.set('Select')
        criterion.valuevar.set('Input')
        criterion.optionvar_a.set('Select')
        criterion.optionvar_b.set('Select')

    criteria[0].radiovar.set(1)
    criteria[1].radiovar.set(5)
    criteria[2].radiovar.set(2)
    criteria[3].radiovar.set(4)
    criteria[4].radiovar.set(2)
    criteria[5].radiovar.set(3)
    criteria[6].radiovar.set(2)
    criteria[7].radiovar.set(4)
    criteria[8].radiovar.set(2)
    criteria[9].radiovar.set(3)

    create_plot()


def temp_func():
    pass


def generate_report():
    report = Report(tabs, criteria)
    report.savePDF()


def create_menu():
    menu = tk.Menu()
    root.config(menu=menu)
    file_menu = tk.Menu(menu)
    file_menu.config(background=colors['background'],
                     activeborderwidth=5,
                     activeforeground=colors['text'],
                     activebackground=colors['accent'],
                     foreground=colors['text'],
                     tearoff=0)

    file_menu.add_command(label='Save image', command=save_image)
    file_menu.add_command(label='Re-set', command=reset_scores)
    file_menu.add_command(label='Report', command=generate_report)
    about_menu = tk.Menu(menu)
    about_menu.config(background=colors['background'],
                      activeborderwidth=5,
                      activeforeground=colors['text'],
                      activebackground=colors['accent'],
                      foreground=colors['text'],
                      tearoff=0)
    menu.add_cascade(label='File', menu=file_menu)
    menu.add_cascade(label='About', menu=about_menu)
    about_menu.add_command(label='Info', command=call_info_popup)


create_menu()

# create the two main frames of the GUI
left_frame = tk.Frame(root, bd=0, padx=2, pady=0, width=300, height=500, bg=colors['background'])
left_frame.pack(side='left', anchor='n')

right_frame = tk.Frame(root, bd=1, padx=2, pady=2, width=500, height=500, bg=colors['foreground'])
right_frame.pack(side='right', anchor='n')


# notebook which will house the 10 tabs (in the left frame)
class Notebook:

    def __init__(self, parent):
        self.parent = parent
        self.notebook = ttk.Notebook(self.parent, height=500, width=300)
        self.notebook.pack()

    def add_tab(self, tab, text):
        self.notebook.add(tab, text=text)


tab_bar = Notebook(left_frame)


class Tab:

    def __init__(self, tab_no, title, text1, text2, criterion):
        self.title = title
        self.parent = tab_bar
        self.frame = tk.Frame(self.parent.notebook, bg=colors['foreground'])
        self.parent.add_tab(self.frame, text=tab_no)
        self.header = ttk.Label(self.frame, text=title, wraplength=280, font=('TkDefaultFont', 10, 'bold'))
        self.header.grid(column=0, row=0, sticky='w')
        self.label1 = ttk.Label(self.frame, text=text1, wraplength=280)
        self.label1.grid(column=0, row=1, sticky='w')
        self.label2 = ttk.Label(self.frame, text=text2, wraplength=280)
        self.label2.grid(column=0, row=2, sticky='w')

        self.criterion = criterion

        self.ratio_label = ttk.Label(self.frame, text='Modify the default weight:')
        self.ratio_label.place(rely=0.91, relx=0.025, anchor='sw')

        self.radio_1 = ttk.Radiobutton(self.frame, text='1', variable=self.criterion.radiovar, command=create_plot,
                                       value=1)
        self.radio_1.place(rely=0.95, relx=0.05, anchor='sw')
        self.radio_2 = ttk.Radiobutton(self.frame, text='2', variable=self.criterion.radiovar, command=create_plot,
                                       value=2)
        self.radio_2.place(rely=0.95, relx=0.15, anchor='sw')
        self.radio_3 = ttk.Radiobutton(self.frame, text='3', variable=self.criterion.radiovar, command=create_plot,
                                       value=3)
        self.radio_3.place(rely=0.95, relx=0.25, anchor='sw')
        self.radio_4 = ttk.Radiobutton(self.frame, text='4', variable=self.criterion.radiovar, command=create_plot,
                                       value=4)
        self.radio_4.place(rely=0.95, relx=0.35, anchor='sw')
        self.radio_5 = ttk.Radiobutton(self.frame, text='5', variable=self.criterion.radiovar, command=create_plot,
                                       value=5)
        self.radio_5.place(rely=0.95, relx=0.45, anchor='sw')


class Criterion:

    def __init__(self, number):
        self.number = number
        self.textvar = tk.StringVar()
        self.textvar.set('1.0')
        self.radiovar = tk.IntVar()
        self.optionvar = tk.StringVar()
        self.optionvar.set('Select')
        self.valuevar = tk.StringVar()
        self.valuevar.set('Input')
        # add optionvars for tab 7:
        self.optionvar_a = tk.StringVar()
        self.optionvar_a.set('Select')
        self.optionvar_b = tk.StringVar()
        self.optionvar_b.set('Select')


# create the 10 criteria values. They can be later called as
# list elements, e.g. to get the value of criterion 3: criteria[2].textvar.get()
criteria = []
for i in range(1, 11):
    criteria.append(Criterion(i))

# assign the default weights (this could be done using itemgetter or itertools.compress, but
# that would require importing additional packages):
criteria[0].radiovar.set(1)
criteria[1].radiovar.set(5)
criteria[2].radiovar.set(2)
criteria[3].radiovar.set(4)
criteria[4].radiovar.set(2)
criteria[5].radiovar.set(3)
criteria[6].radiovar.set(2)
criteria[7].radiovar.set(4)
criteria[8].radiovar.set(2)
criteria[9].radiovar.set(3)


def calculate_score():
    indv_scores = 0
    weights = 0
    for criterion in criteria:
        indv_scores += (float(criterion.textvar.get()) * criterion.radiovar.get())
        weights += criterion.radiovar.get()
    score = indv_scores / weights
    return score


# a function for restarting the right frame (to clear the memory after each change)
# it is then used when there is a need to change the plot
def clear_frame(frame):
    frame.destroy()
    global right_frame
    right_frame = tk.Frame(root, bd=1, padx=2, pady=2, width=500, height=500, bg=colors['foreground'])
    right_frame.pack(side='right', anchor='n')


# connect a float in range 0.0 : 1.0 to a colour in a spectrum from red to yellow to green
# (256^3 discrete colour values):
def colorMapper(value):
    cmap = LinearSegmentedColormap.from_list('rg', ["red", "yellow", "green"], N=256)
    mapped_color = int(value * 255)
    color = cmap(mapped_color)
    return color


def change_dropdown(criterion, tab_choices, *args):
    if criterion.optionvar.get() == 'Select':
        criterion.textvar.set(1.0)
    else:
        criterion.textvar.set(tab_choices[criterion.optionvar.get()])
        create_plot()


def create_plot(event=None):
    # the event=None is passed so that the entry.bind
    # does not return a positional argument;
    # close the current plot window (otherwise the generated plots remain open in the memory):
    plt.close()

    # re-generate the frame:
    clear_frame(right_frame)

    fig = plt.figure(figsize=(3, 3), dpi=150, facecolor=colors['foreground'])
    ax = plt.subplot(projection='polar')
    plt.axis('off')  # hide the axes

    scores = []
    weights = []

    for criterion in criteria:
        scores.append(float(criterion.textvar.get()))
        weights.append(criterion.radiovar.get())

    # just having pi as a variable to avoid importing numpy or math
    pi = 3.141592653589793
    # modify the weights so that the differences between lengths of slices are less extreme:
    weights = [i + 4.5 for i in weights]

    '''
    # generate the thetas once and then just use the values to avoid importing numpy:
    import numpy as np
    theta = np.linspace(0.0, 2 * pi, N, endpoint=False)
    print(theta)
    '''

    theta = [0.0, 0.62831853, 1.25663706, 1.88495559, 2.51327412, 3.14159265, 3.76991118, 4.39822972, 5.02654825,
             5.65486678]
    radii = weights
    width = 2 * pi / len(scores)

    bars = ax.bar(theta, radii, width=width, bottom=0.0, edgecolor='black', linewidth=0.5)
    ax.set_theta_direction(-1)  # theta increasing clockwise
    ax.set_theta_zero_location("N")  # theta=0 at the top

    # annotate the slices with numbers:
    for i in range(10):
        ax.text(theta[i], 4.75, criteria[i].number, verticalalignment='center', horizontalalignment='center',
                color='black', fontsize=6)

    # link the slice (bar) colour with the score:
    for i, bar in zip(range(10), bars):
        bar.set_facecolor(colorMapper(scores[i]))

    # add a Circle artist in the middle to depict the final value:
    circle = plt.Circle((0, 0), radius=4, facecolor=colorMapper(calculate_score()), edgecolor='black',
                        linewidth=0.5, transform=ax.transData._b, )
    plt.gca().add_artist(circle)

    # add the final score in the middle:
    ax.text(0.0, 0.0, str(round(calculate_score(), 2)),
            verticalalignment='center', horizontalalignment='center',
            color='black', fontsize=20)

    canvas = FigureCanvasTkAgg(figure=fig, master=right_frame)
    plot_widget = canvas.get_tk_widget()
    plot_widget.pack(side='top')

    # print('final score: %s' % calculate_score())


# start the GUI with a default plot
create_plot()


class Report:
    def __init__(self, tabs, criteria):
        self.pdf = FPDF('P', 'mm', 'A4')  # 'P' - Portrait
        self.pdf.set_font('Helvetica', '', 10)
        self.pdf.add_page()
        self.pdf.set_margins(left=25, top=20)
        self.tabs = tabs
        self.criteria = criteria
        plt.savefig('temp_figure.png', bbox_inches='tight')
        # insert image (image, x, y, width):
        self.pdf.image('temp_figure.png', 107, 10, 80)
        # delete the temp file from drive:
        os.remove('temp_figure.png')

        self.fill_color = (240, 240, 240)

        # insert title (Helvetica, 'B'old, 14 pt):
        self.pdf.set_font('Helvetica', 'B', 18)
        self.pdf.ln(20)
        self.pdf.cell(100, 12, 'AGREEprep')
        self.pdf.set_font('Helvetica', '', 12)
        self.pdf.ln(10)
        self.pdf.multi_cell(55, 4, 'Analytical Greenness Metric for Sample Preparation')
        self.pdf.ln(10)
        self.pdf.set_font('Helvetica', '', 10)
        self.pdf.cell(100, 12, datetime.now().strftime("%d/%m/%Y %H:%M:%S"))
        self.pdf.ln(30)

        def fieldColor(score):
            x = colorMapper(score)[0] * 255
            y = colorMapper(score)[1] * 255
            z = colorMapper(score)[2] * 255
            self.pdf.set_fill_color(x, y, z)

        def create_header():
            self.pdf.set_font('Helvetica', 'B', 10)
            self.pdf.cell(12, 6, '#', align='C')
            self.pdf.cell(120, 6, 'Criterion', align='L')
            self.pdf.cell(12, 6, 'Score', align='C')
            self.pdf.cell(12, 6, 'Weight', align='C')
            self.pdf.ln(8)
            self.pdf.set_font('Helvetica', '', 10)

        create_header()

        def create_report_field(number, tabs, criterion, text):
            self.pdf.cell(12, 12, number, border=1, align='C')
            self.pdf.set_fill_color(240, 240, 240)
            self.pdf.set_font('Helvetica', 'B', 10)
            self.pdf.cell(120, 6, tabs.title, border=1, ln=2, align='L', fill=True)
            self.pdf.set_font('Helvetica', '', 8)
            self.pdf.cell(120, 6, text, border=1, ln=0, align='L', fill=False)
            self.pdf.set_font('Helvetica', '', 10)

            # modify the position of the following cells to ensure proper alignment:
            x = self.pdf.get_x()
            y = self.pdf.get_y()
            self.pdf.set_xy(x, y - 6)

            # set the colour of the score field:
            fieldColor(float(criterion.textvar.get()))
            self.pdf.cell(12, 12, str(round(float(criterion.textvar.get()), 2)), border=1, fill=True, align='C')
            self.pdf.set_fill_color(240, 240, 240)
            self.pdf.cell(12, 12, str(criterion.radiovar.get()), border=1, fill=True, align='C')
            self.pdf.ln(15)

        create_report_field('1.', self.tabs[0], self.criteria[0],
                            ('Sample preparation placement: ' + self.criteria[0].optionvar.get()))

        create_report_field('2.', self.tabs[1], self.criteria[1],
                            ('Mass [g] or volume [mL] of problematic materials: ' + self.criteria[1].valuevar.get()))

        create_report_field('3.', self.tabs[2], self.criteria[2],
                            (self.criteria[2].optionvar.get()))

        create_report_field('4.', self.tabs[3], self.criteria[3],
                            ('Mass [g] or volume [mL] of waste: ' + self.criteria[3].valuevar.get()))

        create_report_field('5.', self.tabs[4], self.criteria[4],
                            ('Mass [g] or volume [mL] of the sample: ' + self.criteria[4].valuevar.get()))

        create_report_field('6.', self.tabs[5], self.criteria[5],
                            ('Hourly sample throughput: ' + self.criteria[5].valuevar.get()))

        create_report_field('7.', self.tabs[6], self.criteria[6],
                            ('No. of sample prep. steps: ' + self.criteria[6].optionvar_a.get() +
                             '; degree if automation: ' + self.criteria[6].optionvar_b.get()))

        create_report_field('8.', self.tabs[7], self.criteria[7],
                            ('Approximate energy consumption per analysis [W]: ' +
                             self.criteria[7].valuevar.get()))

        create_report_field('9.', self.tabs[8], self.criteria[8],
                            (self.criteria[8].optionvar.get()))

        create_report_field('10.', self.tabs[9], self.criteria[9],
                            ('No. of distinct hazards: ' + self.criteria[9].optionvar.get()))

    def savePDF(self):
        ftypes = [('PDF file', '.pdf'), ('All files', '*')]
        filename = filedialog.asksaveasfilename(filetypes=ftypes, defaultextension='.pdf')
        # save the pdf
        self.pdf.output(filename, 'F')
        # open the file in the system default viewer:
        os.system('start ' + filename)


'''
Populate the tabs
'''
# --------------------------- TAB1 ---------------------------
tab1 = Tab(tab_no='1', title='Sample preparation placement',
           text1='Procedures that avoid excessive sample transportation and '
                 'apply sample preparation that is integrated in analytical procedure are favoured.',
           text2='Sample preparation placement:', criterion=criteria[0])

# create a clickable question mark label that opens a separate mini info page:
label_tab1 = ttk.Label(tab1.frame, text='❓', cursor="hand2", foreground=colors['accent'])
label_tab1.grid(column=0, row=2, padx=170, pady=8, sticky='w')


def call_tab1_popup():
    win_tab1 = tk.Toplevel()
    win_tab1_frame = tk.Frame(win_tab1, width=800, background=colors['foreground'])
    win_tab1_frame.pack()
    win_tab1.wm_title('Sample preparation placement')
    win_tab1.iconbitmap('AGREEPrep_icon.ico')
    text = ttk.Label(win_tab1_frame, text='•    In-line/In situ – sample preparation is carried out in the \n'
                                          '      investigated object. It usually integrates sampling and sample \n'
                                          '      preparation. Good examples can be in-vivo SPME application or \n'
                                          '      the use of passive samplers; \n'
                                          '•    On-line/In situ – sample preparation is performed in situ, \n'
                                          '      sampling and sample preparation are performed in the same \n'
                                          '      place using permanently installed devices with the overall \n'
                                          '      operation being typically fully automated; \n'
                                          '•    On site – sample preparation is performed on site, with the \n'
                                          '      sample preparation device being brought to the sampling site; \n'
                                          '•    Ex situ – sample preparation is performed in the laboratory \n'
                                          '      after sample collection and transportation.',
                     wraplength=400, justify='left')
    text.grid(column=0, row=1, padx=8, pady=8, sticky='w')


label_tab1.bind("<Button-1>", lambda e: call_tab1_popup())

# create a frame to pack the entry into in order to get a border for the entry:
f1 = tk.Frame(tab1.frame, border=2, background=colors['accent'])
f1.grid(column=0, row=3, padx=8, pady=8, sticky='w')

tab1_choices = {'In-line/In situ': 1.0, 'On-line/In situ': 0.66, 'On site': 0.33, 'Ex situ': 0.0}
dropdown1 = ttk.OptionMenu(tab1.frame, criteria[0].optionvar, criteria[0].optionvar.get(), *tab1_choices.keys(),
                           command=lambda x: change_dropdown(criteria[0], tab1_choices))
# the ttk style for OptionMenu does not include a style for the tk dropdown menu,
# so this has to be configured separately:
dropdown1['menu'].config(background=colors['background'],
                         activeborderwidth=5,
                         activebackground=colors['accent'],
                         activeforeground=colors['text'],
                         foreground=colors['text'])

dropdown1.grid(column=0, row=3, padx=8, pady=8, sticky='w')

# --------------------------- TAB2 ---------------------------
tab2 = Tab(tab_no='2', title='Hazardous materials',
           text1='The use of toxic materials, including the use of acids and bases '
                 'for derivatization and digestion, should be avoided.',
           text2='Input mass [g] or volume [mL] of problematic substances:',
           criterion=criteria[1])

label_tab2 = ttk.Label(tab2.frame, text='Include substances that are toxic via one of exposure pathways or '
                                        'labeled as bioaccumulative or persistent.',
                       wraplength=280)
label_tab2.grid(column=0, row=4, sticky='w')

# create a frame to pack the entry into in order to get a border for the entry:
f2 = tk.Frame(tab2.frame, border=2, background=colors['accent'])
f2.grid(column=0, row=3, padx=8, pady=8, sticky='w')
# create the entry and pack it in the frame
entry2 = ttk.Entry(f2, textvariable=criteria[1].valuevar, width=10, style='AGREE_2.0_theme')
entry2.pack()


def calc_crit_2(event=None):
    try:
        if criteria[1].valuevar.get() == 'Input':
            criteria[1].textvar.set(1.0)
        elif float(criteria[1].valuevar.get()) >= 10:
            criteria[1].textvar.set(0)
        elif float(criteria[1].valuevar.get()) <= 0.01:
            criteria[1].textvar.set(1.0)
        else:
            criteria[1].textvar.set(
                abs(-0.145 * log(float(criteria[1].valuevar.get())) + 0.3333))
    except ValueError:
        messagebox.showerror(title='Value error',
                             message='The amount has to be a float or an integer, e.g. 0.14 or 21.')
    create_plot()
    print('criterion 2: ', criteria[1].textvar.get())


entry2.bind('<Return>', calc_crit_2)
ttk.Button(tab2.frame, text='Set', command=calc_crit_2, width=8).grid(column=0, row=3, padx=80, pady=8, sticky='w')

# --------------------------- TAB3 ---------------------------
tab3 = Tab(tab_no='3', title='Sustainability, renewability, and reusability of materials',
           text1='The application of materials from sustainable and renewable sources '
                 'should be prioritized.',
           text2='Ratio of the mass of sustainable and renewable materials to the total '
                 'mass of materials used:', criterion=criteria[2])

tab3_choices = {'Only sustainable and renewable materials are used SEVERAL TIMES': 1.0,
                '> 75% of reagents and materials are sustainable or renewable': 0.75,
                '50-75% of reagents and materials are sustainable or renewable, but can only be used ONCE': 0.50,
				'Materials are not sustainable or renewable, but are used SEVERAL TIMES': 0.50,
                '25-50% of reagents and materials are sustainable or renewable': 0.25,
                '< 25% of reagents and materials are sustainable or renewable, but can only be used ONCE': 0,}
dropdown3 = ttk.OptionMenu(tab3.frame, criteria[2].optionvar, criteria[2].optionvar.get(), *tab3_choices.keys(),
                           command=lambda x: change_dropdown(criteria[2], tab3_choices))
# the ttk style for OptionMenu does not include a style for the tk dropdown menu,
# so this has to be configured separately:
dropdown3['menu'].config(background=colors['background'],
                         activeborderwidth=5,
                         activebackground=colors['accent'],
                         activeforeground=colors['text'],
                         foreground=colors['text'])

dropdown3.grid(column=0, row=3, padx=8, pady=8, sticky='w')

# --------------------------- TAB4 ---------------------------
tab4 = Tab(tab_no='4', title='Waste',
           text1='In analytical chemistry, all material inputs, including '
                 'the sample preparation stage, can be treated as waste.',
           text2='Input mass [g] or volume [mL] of waste:', criterion=criteria[3])
# create a frame to pack the entry into in order to get a border for the entry:
f4 = tk.Frame(tab4.frame, border=2, background=colors['accent'])
f4.grid(column=0, row=3, padx=8, pady=8, sticky='w')
# create the entry and pack it in the frame
entry4 = ttk.Entry(f4, textvariable=criteria[3].valuevar, width=10, style='AGREE_2.0_theme')
entry4.pack()


def calc_crit_4(event=None):
    try:
        if criteria[3].valuevar.get() == 'Input':
            criteria[3].textvar.set(1.0)
        elif float(criteria[3].valuevar.get()) >= 50:
            criteria[3].textvar.set(0)
        elif float(criteria[3].valuevar.get()) <= 0.1:
            criteria[3].textvar.set(1.0)
        else:
            criteria[3].textvar.set(
                abs(-0.161 * log(float(criteria[3].valuevar.get())) + 0.6295))
    except ValueError:
        messagebox.showerror(title='Value error',
                             message='The amount has to be a float or an integer, e.g. 0.14 or 21.')
    create_plot()
    print('Criterion 4: ', criteria[3].textvar.get())


entry4.bind('<Return>', calc_crit_4)
ttk.Button(tab4.frame, text='Set', command=calc_crit_4, width=8).grid(column=0, row=3, padx=80, pady=8, sticky='w')

# --------------------------- TAB5 ---------------------------
tab5 = Tab(tab_no='5', title='Size economy of the sample',
           text1='Smaller sample sizes should be favoured, as long as sample representativeness is assured.',
           text2='Input mass [g] or volume [mL] of the sample:',
           criterion=criteria[4])

f5 = tk.Frame(tab5.frame, border=2, background=colors['accent'])
f5.grid(column=0, row=3, padx=8, pady=8, sticky='w')
entry5 = ttk.Entry(f5, textvariable=criteria[4].valuevar, width=10, style='AGREE_2.0_theme')
entry5.pack()


def calc_crit_5(event=None):
    try:
        if criteria[4].valuevar.get() == 'Input':
            criteria[4].textvar.set(1.0)
        elif float(criteria[4].valuevar.get()) >= 100:
            criteria[4].textvar.set(0)
        elif float(criteria[4].valuevar.get()) <= 0.1:
            criteria[4].textvar.set(1.0)
        else:
            criteria[4].textvar.set(
                abs(-0.145 * log(float(criteria[4].valuevar.get())) + 0.6667))
    except ValueError:
        messagebox.showerror(title='Value error',
                             message='The amount has to be a float or an integer, e.g. 0.14 or 21.')
    create_plot()


entry5.bind('<Return>', calc_crit_5)
ttk.Button(tab5.frame, text='Set', command=calc_crit_5, width=8).grid(column=0, row=3, padx=80, pady=8, sticky='w')

# --------------------------- TAB6 ---------------------------
tab6 = Tab(tab_no='6', title='Sample throughput',
           text1='Sample throughput determines the overall duration of the sample preparation stage.',
           text2='Input the number of samples that can be prepared in one hour:',
           criterion=criteria[5])

f6 = tk.Frame(tab6.frame, border=2, background=colors['accent'])
f6.grid(column=0, row=3, padx=8, pady=8, sticky='w')
entry6 = ttk.Entry(f6, textvariable=criteria[5].valuevar, width=10, style='AGREE_2.0_theme')
entry6.pack()


def calc_crit_6(event=None):
    try:
        if criteria[5].valuevar.get() == 'Input':
            criteria[5].textvar.set(1.0)
        elif float(criteria[5].valuevar.get()) >= 70:
            criteria[5].textvar.set(1.0)
        elif float(criteria[5].valuevar.get()) <= 1.0:
            criteria[5].textvar.set(0.0)
        else:
            criteria[5].textvar.set(
                abs(0.2354 * log(float(criteria[5].valuevar.get()))))
    except ValueError:
        messagebox.showerror(title='Value error',
                             message='The amount has to be a float or an integer, e.g. 0.14 or 21.')
    create_plot()


entry6.bind('<Return>', calc_crit_6)
ttk.Button(tab6.frame, text='Set', command=calc_crit_6, width=8).grid(column=0, row=3, padx=80, pady=8, sticky='w')

# --------------------------- TAB7 ---------------------------
tab7 = Tab(tab_no='7', title='Integration and automation',
           text1='Sample preparation should be simplified and automated if possible.',
           text2='1. Select the number of discrete steps in the sample preparation procedure:',
           criterion=criteria[6])

tab7a_choices = {'2 steps or fewer': 1.0,
                 '3 steps': 0.75,
                 '4 steps': 0.50,
                 '5 steps': 0.25,
                 '6 steps or more': 0}


def calc_crit_7(event=None):
    # create temporary variables for each dropdown:
    a = tk.StringVar()
    b = tk.StringVar()
    # reset the temp variables to default if the global reset was triggered:
    a.set('1.0')
    b.set('1.0')

    if criteria[6].optionvar_a.get() != 'Select':
        a.set(tab7a_choices[criteria[6].optionvar_a.get()])

    if criteria[6].optionvar_b.get() != 'Select':
        b.set(tab7b_choices[criteria[6].optionvar_b.get()])

    criteria[6].textvar.set(str(float(a.get()) * float(b.get())))
    create_plot()


dropdown7a = ttk.OptionMenu(tab7.frame, criteria[6].optionvar_a, criteria[6].optionvar_a.get(), *tab7a_choices.keys(),
                            command=calc_crit_7)

# the ttk style for OptionMenu does not include a style for the tk dropdown menu,
# so this has to be configured separately:
dropdown7a['menu'].config(background=colors['background'],
                          activeborderwidth=5,
                          activebackground=colors['accent'],
                          activeforeground=colors['text'],
                          foreground=colors['text'])

dropdown7a.grid(column=0, row=3, padx=8, pady=8, sticky='w')

label_tab7 = ttk.Label(tab7.frame, text='2. Select the degree of automation:',
                       wraplength=280)
label_tab7.grid(column=0, row=4, sticky='w')

tab7b_choices = {'Fully automated systems': 1.0,
                 'Semi-automated systems': 0.50,
                 'Manual systems': 0.25}

dropdown7b = ttk.OptionMenu(tab7.frame, criteria[6].optionvar_b, criteria[6].optionvar_b.get(), *tab7b_choices.keys(),
                            command=calc_crit_7)

dropdown7b['menu'].config(background=colors['background'],
                          activeborderwidth=5,
                          activebackground=colors['accent'],
                          activeforeground=colors['text'],
                          foreground=colors['text'])

dropdown7b.grid(column=0, row=5, padx=8, pady=8, sticky='w')

# --------------------------- TAB8 ---------------------------
# remember to include a drop-down list of generic requirements for typical devices!
tab8 = Tab(tab_no='8', title='Energy consumption',
           text1='The power consumption per analysis should be minimized.',
           text2='Input the energy consumption [Wh] per sample, accounting for the sample throughput:',
           criterion=criteria[7])

f8 = tk.Frame(tab8.frame, border=2, background=colors['accent'])
f8.grid(column=0, row=3, padx=8, pady=8, sticky='w')
entry8 = ttk.Entry(f8, textvariable=criteria[7].valuevar, width=10, style='AGREE_2.0_theme')
entry8.pack()


def calc_crit_8(event=None):
    try:
        if criteria[7].valuevar.get() == 'Input':
            criteria[7].textvar.set(1.0)
        elif float(criteria[7].valuevar.get()) >= 500:
            criteria[7].textvar.set(0)
        elif float(criteria[7].valuevar.get()) <= 10:
            criteria[7].textvar.set(1.0)
        else:
            criteria[7].textvar.set(
                abs(-0.256 * log(float(criteria[7].valuevar.get())) + 1.5886))
    except ValueError:
        messagebox.showerror(title='Value error',
                             message='The amount has to be a float or an integer, e.g. 0.14 or 21.')
    create_plot()


entry8.bind('<Return>', calc_crit_8)
ttk.Button(tab8.frame, text='Set', command=calc_crit_8, width=8).grid(column=0, row=3, padx=80, pady=8, sticky='w')

# --------------------------- TAB9 ---------------------------
tab9 = Tab(tab_no='9', title='Post-sample preparation configuration for analysis',
           text1='The prepared sample matrix should be compatible with the analytical instrument, '
                 'which in turn affects the overall environmental impact of the procedure.',
           text2='Select the final determination technique or its closest analogue:',
           criterion=criteria[8])

tab9_choices = {'Simple, readily available detection: smartphones, desktop scanners, paper strips, etc.': 1.0,
                'Spectrophotometry, surface analysis techniques, voltammetry, potentiometry, etc.': 0.75,
                'GC with non-MS detection, atomic absorption spectroscopy, capillary electrophoresis, etc.': 0.50,
                'Liquid chromatography, gas chromatography with quadrupole detection, etc.': 0.25,
                'Advanced MS with high energy and/or noble gas consumption: ICP-OES, ICP-MS, etc.': 0}

dropdown9 = ttk.OptionMenu(tab9.frame, criteria[8].optionvar, criteria[8].optionvar.get(), *tab9_choices.keys(),
                           command=lambda x: change_dropdown(criteria[8], tab9_choices))
# the ttk style for OptionMenu does not include a style for the tk dropdown menu,
# so this has to be configured separately:
dropdown9['menu'].config(background=colors['background'],
                         activeborderwidth=5,
                         activebackground=colors['accent'],
                         activeforeground=colors['text'],
                         foreground=colors['text'])

dropdown9.grid(column=0, row=3, padx=8, pady=8, sticky='w')

# --------------------------- TAB10 ---------------------------
tab10 = Tab(tab_no='10', title='Operator\'s safety',
            text1='Hazards associated with the procedure, including physical ones, should '
                  'be minimized.',
            text2='The number of distinct hazards of chemical (threats indicated in pictograms '
                  'labelling used chemicals) and physical nature:',
            criterion=criteria[9])

tab10_choices = {'No hazards or no exposure': 1.0,
                 '1 hazard': 0.75,
                 '2 hazards': 0.50,
                 '3 hazards': 0.25,
                 '4 or more hazards': 0}
dropdown10 = ttk.OptionMenu(tab10.frame, criteria[9].optionvar, criteria[9].optionvar.get(), *tab10_choices.keys(),
                            command=lambda x: change_dropdown(criteria[9], tab10_choices))
# the ttk style for OptionMenu does not include a style for the tk dropdown menu,
# so this has to be configured separately:
dropdown10['menu'].config(background=colors['background'],
                          activeborderwidth=5,
                          activebackground=colors['accent'],
                          activeforeground=colors['text'],
                          foreground=colors['text'])

dropdown10.grid(column=0, row=3, padx=8, pady=8, sticky='w')

'''
End of the tabs
'''

# Create a list of the tabs and generate the report:
tabs = [tab1, tab2, tab3, tab4, tab5, tab6, tab7, tab8, tab9, tab10]


def main():
    while True:
        root.mainloop()


main()
